//===----------- ThreeAddr2TwoAddr_Fer.cpp - Printing prototype -----------===//
//
//             Register Allocation via Coloring of Chordal Graphs 
//
// This pass converts a three address instruction to a two address instruction.
//
// Date: May 28th, 2006
//
//===----------------------------------------------------------------------===//
//
// In LLVM, all the instructions are three address code, but some target
// architectures have two address instructions. Therefore, it is necessary to
// make this conversion, what is exaclty what this class does.
//
//===----------------------------------------------------------------------===//

#ifndef THREE_ADDR_2_TWO_ADDR_FER
#define THREE_ADDR_2_TWO_ADDR_FER

namespace llvm {

    class ThreeAddr2TwoAddr_Fer : public MachineFunctionPass {

        public:
 
            /// reporst that this pass does not change anything on the control
            /// flow graph.
            virtual void getAnalysisUsage(AnalysisUsage &AU) const {
                AU.setPreservesAll();
            }

            /// Informs the name of the pass, so it can appear in tools like
            /// llc.
            virtual const char * getPassName() const {
                return "Three to two address convertion - Fernando.";
            }

        private:
 
            // Checks each instruction in the function, and converts the three
            // address instructions to two address if necessary.
            virtual bool runOnMachineFunction(MachineFunction & mf);

            // The function that is been compiled.
            MachineFunction * mf;

            // Information about the set of available registers.
            const MRegisterInfo * ri;

            // Information about the instructions. Notice that the current
            // implementation of this class is very limited in the sense that
            // it expects a very particular type of three address instruction.
            const TargetInstrInfo * tii;

            // This method takes an instruction in three address code format,
            // and converts it two two instructions: a copy, and a second
            // instruction in two address code format.
            // Ex.: a := b + c => a := b; a += c;
            void convert_to_2_addr_plus_copy(
                MachineBasicBlock::iterator mii,
                MachineBasicBlock & mbb
            ); 

            // Finds the two address instructions in the basic block and
            // converts them to two address instructions. We assume that, if
            // the instruction is in the format a := b + a, then it is
            // commutative. We also assume that the first three operands of the
            // instruction are registers.
            bool remove3address(MachineBasicBlock & mbb);
    };

}

#endif
