//===----------------- Util_Fer.cpp - Layout output ------------------===//
//
//              Register Allocation via Coloring of Chordal Graphs 
//
// This class contains some handy functions to retrieve data from instructions,
// operands, basic blocks, etc.
//
// Date: September 9th, 2006
//
//===----------------------------------------------------------------------===//

#include "llvm/CodeGen/MachineInstr.h"

namespace llvm {

#ifndef UTIL_FER_H
#define UTIL_FER_H
 
class Util_Fer {

public:

//===--------------------------------------------------------------------------
// This method returns true if the instruction uses a register operand equal
// to reg, otherwise it returns false.
//===--------------------------------------------------------------------------
static bool contains_use(const MachineInstr & mi, const unsigned reg) {
    unsigned i = 0;
    while(i != mi.getNumOperands()) {
        const MachineOperand & mo = mi.getOperand(i);
        if (mo.isRegister() && mo.getReg() && mo.isUse()) {
            if(mo.getReg() == reg) {
                return true;
            }
        }
        i++;
    }
    return false;
}


//===--------------------------------------------------------------------------
// This method returns true if the instruction uses a register operand equal
// to reg, otherwise it returns false.
//===--------------------------------------------------------------------------
static bool contains_def(const MachineInstr & mi, const unsigned reg) {
    unsigned i = 0;
    while(i != mi.getNumOperands()) {
        const MachineOperand & mo = mi.getOperand(i);
        if (mo.isRegister() && mo.getReg() && mo.isDef()) {
            if(mo.getReg() == reg) {
                return true;
            }
        } else if (mo.isRegister() && mo.getReg() && mo.isUse()) {
            return false;
        }
        i++;
    }
    return false;
}


//===--------------------------------------------------------------------------
// This method returns the position of the operand in the instruction. For
// instance, a := b + c, pos(a) == 0, pos(b) == 1, and pos(c) == 2.
//===--------------------------------------------------------------------------
static bool operand_position(
    const MachineInstr & mi,
    const unsigned reg,
    const unsigned pos
) {
    if(pos >= mi.getNumOperands()) {
        return false;
    } else {
        const MachineOperand & mo = mi.getOperand(pos);
        return mo.isRegister() && mo.getReg() && mo.getReg() == reg;
    }
}

//===--------------------------------------------------------------------------
// Returns true if the given instruction is two address code, and false
// otherwise.
//===--------------------------------------------------------------------------
static bool is_two_address(const MachineInstr & mi, MachineFunction & mf) {
    const TargetMachine & target_machine = mf.getTarget();
    const TargetInstrInfo & tii = * (target_machine.getInstrInfo());
    return tii.isTwoAddrInstr(mi.getOpcode());
}

private:

};

#endif

}
