//===----------- GraphFer_Fer.cpp - Bank of Machine Registers -------------===//
//
//              Register Allocation via Coloring of Chordal Graphs 
//
// This class implements a undirected graph.
//
// Date: March 2nd 2007
//
//===----------------------------------------------------------------------===//
//
//
//===----------------------------------------------------------------------===//

#include <map>
#include <iostream>

#include "llvm/Function.h"
#include "llvm/Instructions.h"                   // MachineOperand, etc
#include "llvm/CodeGen/MachineFunction.h"

#include "llvm/fernando/graph/Node_Fer.h"
#include "llvm/fernando/graph/Graph_Fer.h"

using namespace llvm;


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
void Graph_Fer::connect(unsigned v, unsigned u) {
    if(! this->contains(v)) {
        Node_Fer * n = new Node_Fer(v);
        this->nodes[v] = n;
    }
    if(! this->contains(u)) {
        Node_Fer * n = new Node_Fer(u);
        this->nodes[u] = n;
    }
    if(v != u) {
        Node_Fer * nv = this->nodes[v];
        Node_Fer * nu = this->nodes[u];
        nv->link(nu);
        nu->link(nv);
    }
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
bool Graph_Fer::are_adj(unsigned u, unsigned v) {
    Node_Fer * nv = this->nodes[v];
    Node_Fer * nu = this->nodes[u];
    if(nv != NULL && nu != NULL)  {
        if(nv->is_neighbor(nu)) {
            assert(nu->is_neighbor(nv) && "Single side adjacency.");
            return true;
        }
    }
    return false;
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
void Graph_Fer::remove(unsigned u) {
    Node_Fer * n = this->nodes[u];
    n->unlink_all();
    this->nodes.erase(u);
    delete n;
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
void print_header(MachineFunction & mf) {
    const Function * f = mf.getFunction();
    std::cerr << "Graph " << f->getName() << "{\n\n";

    // print the graph attributes:
    std::cerr << "\tgraph [\n";
    std::cerr << "\t\tconcentrate = \"true\"\n";
    std::cerr << "\t];\n\n";

    // prints he format of each node.
    std::cerr << "\tnode [\n";
    std::cerr << "\t\tfontname = \"Courier\"\n";
    std::cerr << "\t\tfontsize = \"10\"\n";
    std::cerr << "\t\tshape = \"box\"\n";
    std::cerr << "\t];\n\n";

    // prints he format of each edge.
    std::cerr << "\tedge [\n";
    std::cerr << "\t\tfontname = \"Courier\"\n";
    std::cerr << "\t\tfontsize = \"8\"\n";
    std::cerr << "\t];\n\n";
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
const void Graph_Fer::dump(MachineFunction & mf) const {
    print_header(mf);
    std::map< unsigned, Node_Fer * >::const_iterator it = this->nodes.begin();
    while(it != nodes.end()) {
        const Node_Fer * n = (*it).second;
        std::cerr << n->to_string(mf);
        it++;
    }
    std::cerr << "}\n";
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
void Graph_Fer::clear() {
    while(this->nodes.size() > 0) {
        std::map< unsigned, Node_Fer * >::iterator it = this->nodes.begin();
        Node_Fer * n = (*it).second;
        this->nodes.erase(it);
        delete n;
    }
    this->nodes.clear();
}
