//===---------------- Node_Fer.cpp - Printing prototype -------------------===//
//
//              Register Allocation via Coloring of Chordal Graphs 
//
// This is the node of a graph.
//
// Date: May 27th, 2006
//
//===----------------------------------------------------------------------===//
//
//===----------------------------------------------------------------------===//

#include <set>

#include "llvm/CodeGen/MachineFunctionPass.h"    // MachineBasicBlock
#include "llvm/Target/MRegisterInfo.h"

#include "llvm/fernando/NeatPrint_Fer.h"
#include "llvm/fernando/graph/Node_Fer.h"

using namespace llvm;

//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
bool Node_Fer::link(Node_Fer * n) {
   if(this->neighbors.find(n) == this->neighbors.end()) {
       this->neighbors.insert(n);
       return true;
   }
   return false;
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
bool Node_Fer::unlink(Node_Fer * n) {
    bool ret_val = this->neighbors.find(n) != this->neighbors.end();
    this->neighbors.erase(n);
    return ret_val;
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
void Node_Fer::unlink_all() {
    std::set<Node_Fer *>::iterator pos;
    for (pos = neighbors.begin(); pos != neighbors.end(); ++pos) {
        Node_Fer * n = * pos;
        n->unlink(this);
    }
    this->neighbors.clear();
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
const std::set<Node_Fer *> & Node_Fer::get_neighbors() const {
    return this->neighbors;
}


//===--------------------------------------------------------------------------
//
//===--------------------------------------------------------------------------
const std::string Node_Fer::to_string(const MachineFunction & mf) const {
    std::ostringstream str_aux;
    str_aux << "";
    std::set<Node_Fer *>::const_iterator it = this->neighbors.begin();
    while(it != this->neighbors.end()) {
        const Node_Fer * n = *it;
        str_aux << "\t\"" << NeatPrint_Fer::op2string(n->get_value(), mf) <<
                   "\" -- \"" << NeatPrint_Fer::op2string(value, mf) << "\"\n";
        it++;
    }
    return str_aux.str();
}
