//===-- StatsCollector_Fer.cpp - Collects statistics about instructions ---===//
//
//                     The LLVM Compiler Infrastructure
//
// Author: Fernando Magno Quintao Pereira
//
//===----------------------------------------------------------------------===//
//
// This pass collects some statistics about the target program.
//
//===----------------------------------------------------------------------===//

#include <iostream>

#include "llvm/CodeGen/Passes.h"
#include "llvm/CodeGen/MachineFunctionPass.h"
#include "llvm/CodeGen/MachineInstr.h"
#include "llvm/CodeGen/SSARegMap.h"
#include "llvm/Target/TargetInstrInfo.h"
#include "llvm/Target/TargetMachine.h"
#include "llvm/ADT/Statistic.h"
#include "llvm/Support/Compiler.h"

#include "llvm/fernando/StatsCollector_Fer.h"

using namespace llvm;

namespace {
    Statistic<> numBasicB("numBasicB", "Total number of basic blocks.");
    Statistic<> numInstr("numInsts", "Total number of instructions.");
    Statistic<> numPreCol("numPreCol",
                                    "Number of instr containing precoloreds.");

    RegisterPass<StatsCollector_Fer>
    X("stats_col", "Collector of statistic data");
}

//===--------------------------------------------------------------------------
// Collects statistic information about each basic block of the control flow
// graph.
//===--------------------------------------------------------------------------
bool StatsCollector_Fer::runOnMachineFunction(MachineFunction & mf) {
    // traverse the control flow graph in depth first order pinpointing the
    // registers alive in each edge:
    const TargetMachine & target_machine = mf.getTarget();
    const TargetInstrInfo & tii = * (target_machine.getInstrInfo());
    for (MachineFunction::iterator mbbi = mf.begin(), mbbe = mf.end();
                                                        mbbi != mbbe; ++mbbi) {
        numBasicB++;
        MachineBasicBlock * mbb = mbbi;
        MachineBasicBlock::const_iterator mii = mbb->begin();
        while (mii != mbb->end()) {
            numInstr++;
            const MachineInstr * mi = mii;
            const TargetInstrDescriptor & tid = tii.get(mi->getOpcode());
            if(tid.ImplicitUses || tid.ImplicitDefs) {
                numPreCol++;
            } else {
                for(unsigned u = 0; u < mi->getNumOperands(); u++) {
                    const MachineOperand & mo = mi->getOperand(u);
                    if(mo.isRegister() && mo.getReg()) {
                        if(MRegisterInfo::isPhysicalRegister(mo.getReg())) {
                            numPreCol++;
                            break;
                        }
                    }
                }
            }
            mii++;
        }
    }
    return false;
}
